import os
from . import config
import typing
import time
import requests
from . import utils
import json
from . import const

tokens = {}

TOKEN_DECAY_SECONDS = 15*60

def get_auth_header(resource_id):
  """
  获取session
  """
  if os.environ.get("DEBUG") == "true":
    return {"X-Jwt-Data": "{"+f'"project_id":"project_a", "resource_id":"{config.get_baas_file_resource_id()}"'+"}"}
  else:
    if resource_id not in tokens or time.time()>tokens[resource_id][1]:
      refresh_token(resource_id)
    return {"X-Baas-Token": tokens[resource_id][0]}


@utils.retry(2, 0.1)
def refresh_token(resource_id):
  """
  刷新token
  """
  resource = config.get_resource_info_by_id(resource_id)
  resp = requests.post(_get_token_url(),json={
      "ResourceID": resource.id,
      "Secret": resource.secret
    },
    headers={
      "Content-Type": "application/json"
    },
    timeout=(1, 10))
  token = utils._parse_response(resp)
  expireAt = time.time()+token["ExpireSecond"] - TOKEN_DECAY_SECONDS
  tokens[resource_id] = (token["Token"], expireAt)

def _get_token_url():
  return f"{config.get_baas_server_host()}/v1/baas/control/get_token"


import sys
import os
sys.path = [os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "src"))] + sys.path
import a0_baas_sdk.file as file
import a0_baas_sdk.remote.auth as auth
import a0_baas_sdk.remote.utils as utils
import requests

@utils.retry(0, 1)
def func():
  print("excuted")
  raise Exception()

if __name__ == "__main__":
  f = open("blob", "wb")
  f.write(bytes(1024*1024*30))
  f.close()
  fd = file.upload("blob", open("blob", "rb"), timeout=(10, 10))
  print(fd.url)
  print(fd.id)
  resp = file.download(fd.id,timeout=(1, 10))
  print("download success")
  file.delete(fd.id,timeout=(1, 10))
  try:
    file.download(fd.id, timeout=(1, 10))
  except file.FileNotFound:
    print("can not download deleted file!")



import os
import json
import traceback
from . import const
from typing import List as _List


def get_baas_server_host():
  return os.environ.get("baas_host")

def get_baas_redis_server_host():
  return os.environ.get("baas_redis_host")

def get_baas_resource_conf():
  return os.environ.get(const.RESOURCE_ENVIRON_KEY)

def get_baas_file_resource_id():
  cnf = os.environ.get(const.RESOURCE_ENVIRON_KEY)
  resource = json.loads(cnf)
  for r in resource:
    if r["Type"]==const.RESOURCE_TYPE_FILE:
      return r["ResourceID"]
  return None

def get_baas_api_timeout(api:str):
  try:
    cnf = os.environ.get("BaaSAPITimeout")
    timeouts = json.load(cnf)
    return timeouts[api]
  except Exception as e:
    # set default to 10s
    traceback.print_exc()
    return 10

def get_s3_connect_timeout()->int:
  return 10

def _init():
  global resource_info
  resource_info = []
  from . import config
  import json
  cnf = config.get_baas_resource_conf()
  resource_configs = json.loads(cnf)
  for resource in resource_configs:
    resource_info.append(_Resource(
      resource["Type"],
      resource["ResourceID"],
      resource["Secret"]
    ))
class _Resource(object):
  resource_type:str = None
  secret:str = None
  id:str = None

  def __init__(self, resource_type, id, secret) -> None:
    self.id = id
    self.secret = secret
    self.resource_type = resource_type

def get_resource_info_by_id(id:str)->_Resource:
  for resource in resource_info:
    if resource.id == id:
      return resource
  return None

_init()


def get_baas_redis_resource_arr() -> _List[_Resource]:
  arr = []
  for r in resource_info:
    if r.resource_type==const.RESOURCE_TYPE_REDIS:
      arr.append(r)
  return arr


import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__),"../src"))

import a0_baas_sdk.redis as redis_module
# from a0_baas_sdk.redis import redis as redis_module


# result = redis_module.redis.hget("pythonsdk_hset002", "f04")
# result = redis_module.redis_clients["pysdk_resource_id_02"].hget("pythonsdk_hset002", "f02")

# result = redis_module.redis_clients["pysdk_resource_id_02"].hgetall("pythonsdk_hset002")

# result = redis_module.redis_clients["pysdk_resource_id_02"].hincrby("pythonsdk_hset002", "some2", 211)

# result = redis_module.redis_clients["pysdk_resource_id_02"].hincrbyfloat("pythonsdk_hset002", "some2", 1.211)

# result = redis_module.redis_clients["pysdk_resource_id_02"].hkeys("pythonsdk_hset002")

# result = redis_module.redis_clients["pysdk_resource_id_02"].hlen("pythonsdk_hset0021")

# result = redis_module.redis_clients["pysdk_resource_id_02"].hmget("pythonsdk_hset002", "some1", "some")

# result = redis_module.redis_clients["pysdk_resource_id_02"].hmset("pythonsdk_hset002", "some3", "v3", "some4", "v4")

# result = redis_module.redis_clients["pysdk_resource_id_02"].hvals("pythonsdk_hset002")

# result = redis_module.redis_clients["pysdk_resource_id_02"].lindex("pythonsdk_list002", 123)

# result = redis_module.redis_clients["pysdk_resource_id_02"].llen("pythonsdk_list002")

# result = redis_module.redis_clients["pysdk_resource_id_02"].lpop("pythonsdk_list002")

# result = redis_module.redis_clients["pysdk_resource_id_02"].rpop("pythonsdk_list0021")

# result = redis_module.redis_clients["pysdk_resource_id_02"].rpoplpush("xxx:\{pythonsdk_list0022\}", "xxx:\{pythonsdk_list\}")

# result = redis_module.redis_clients["pysdk_resource_id_02"].lpushx("pythonsdk_list008", "qq")

# result = redis_module.redis_clients["pysdk_resource_id_02"].rpushx("pythonsdk_list008", "pp")

# result = redis_module.redis_clients["pysdk_resource_id_02"].lpush("pythonsdk_list002", "f","g")

# result = redis_module.redis_clients["pysdk_resource_id_02"].rpush("pythonsdk_list008", "f","g")

# result = redis_module.redis_clients["pysdk_resource_id_02"].llen("pythonsdk_list008")

# result = redis_module.redis_clients["pysdk_resource_id_02"].lrange("pythonsdk_list008", 0,-1)

# result = redis_module.redis_clients["pysdk_resource_id_02"].srandmember("pythonsdk_set008", 123)

# result = redis_module.redis_clients["pysdk_resource_id_02"].lset("pythonsdk_list002", 3, "aaa")
# result = redis_module.redis_clients["pysdk_resource_id_02"].hset("pythonsdk_hset002", "some5", "v5")
# result = redis_module.redis_clients["pysdk_resource_id_02"].hsetnx("pythonsdk_hset002", "some6", "v7")
# result = redis_module.redis_clients["pysdk_resource_id_02"].ltrim("pythonsdk_list002", 0, 3)


# result = redis_module.redis_clients["pysdk_resource_id_02"].get("pythonsdk_get001")
# result = redis_module.redis_clients["pysdk_resource_id_02"].set("pythonsdk_get009", "val0018", "xx")

# result = redis_module.redis_clients["pysdk_resource_id_02"].set("pythonsdk_get0099", "val0019", "nx")

# result = redis_module.redis_clients["pysdk_resource_id_02"].delete()

# result = redis_module.redis_clients["pysdk_resource_id_02"].exists("pythonsdk_get001")

# result = redis_module.redis_clients["pysdk_resource_id_02"].mget

# result = redis_module.redis_clients["pysdk_resource_id_02"].set("setkeyfor4param", "valuefor4param", "px", 10000,"xx")

# result = redis_module.redis_clients["pysdk_resource_id_02"].get("setkeyfor4param")

# result = redis_module.redis_clients["pysdk_resource_id_02"].mset("msetkey001", "msetvalue001", "msetkey001","msetkey002")

# result = redis_module.redis_clients["pysdk_resource_id_02"].mset("user:{user1}", "value1", "user:{user2}", "value2", "user:{user3}","value3")

# result = redis_module.redis_clients["pysdk_resource_id_02"].get("msetkey001")

# result = redis_module.redis_clients["pysdk_resource_id_02"].setbit("msetkey001", 100, 1)

# result = redis_module.redis_clients["pysdk_resource_id_02"].getbit("msetkey0011", 110)

# result = redis_module.redis_clients["pysdk_resource_id_02"].strlen("msetkey001")


# result = redis_module.redis_clients["pysdk_resource_id_02"].hdel("pythonsdk_hset002", "qq","qqqs")

# result = redis_module.redis_clients["pysdk_resource_id_02"].sadd("pythonsdk_set002", "one","two","one", "two")

# result = redis_module.redis_clients["pysdk_resource_id_02"].sadd("pythonsdk_set003", "one","two","three")

# result = redis_module.redis_clients["pysdk_resource_id_02"].sdiff("pythonsdk_set003", "pythonsdk_set002")

# result = redis_module.redis_clients["pysdk_resource_id_02"].sadd("prefix:\{pythonsdk_set002\}", "one","two","one", "two")

# result = redis_module.redis_clients["pysdk_resource_id_02"].sadd("prefix:\{pythonsdk_set003\}", "one","two","three")

# result = redis_module.redis_clients["pysdk_resource_id_02"].sdiff("prefix:\{pythonsdk_set002\}")

# result = redis_module.redis_clients["pysdk_resource_id_02"].sismember("pythonsdk_set0031", "one1")

# result = redis_module.redis_clients["pysdk_resource_id_02"].smembers("pythonsdk_set003")

# result = redis_module.redis_clients["pysdk_resource_id_02"].smove("pythonsdk_set003", "pythonsdk_set002", "one")

# result = redis_module.redis_clients["pysdk_resource_id_02"].smembers("pythonsdk_set003")

# result = redis_module.redis_clients["pysdk_resource_id_02"].sadd("\{pythonsdk_set002\}:qq", "one","two","zero","two")

# result = redis_module.redis_clients["pysdk_resource_id_02"].sadd("\{pythonsdk_set002\}:pp", "one","two","three")

# result = redis_module.redis_clients["pysdk_resource_id_02"].sdiff("\{pythonsdk_set002\}:pp1", "\{pythonsdk_set002\}:qq1")


# result = redis_module.redis_clients["pysdk_resource_id_02"].zadd("\{pythonsdk_z002\}:001", 100, "val1")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zadd("\{pythonsdk_z002\}:002", 100.11, "val2")
# result = redis_module.redis_clients["pysdk_resource_id_02"].zadd("\{pythonsdk_z002\}:002", 100.21, "val1")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zinterstore("\{pythonsdk_z002\}:003", 2, "\{pythonsdk_z002\}:001", "\{pythonsdk_z002\}:002")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zrange("\{pythonsdk_z002\}:003", 0, -1, 'withscores')

# result = redis_module.redis_clients["pysdk_resource_id_02"].zunionstore("\{pythonsdk_z002\}:004", 2, "\{pythonsdk_z002\}:001", "\{pythonsdk_z002\}:002")
# result = redis_module.redis_clients["pysdk_resource_id_02"].zrange("\{pythonsdk_z002\}:004", 0, -1, 'withscores')

# result = redis_module.redis_clients["pysdk_resource_id_02"].zcount("\{pythonsdk_z002\}:004", "0", "250")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zincrby("\{pythonsdk_z002\}:004", 2.5, "val1")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zrangebyscore("\{pythonsdk_z002\}:004", "0", "150")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zrank("\{pythonsdk_z002\}:004", "val1")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zrank("\{pythonsdk_z002\}:004", "val1")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zadd("\{pythonsdk_z002\}:004", 700.21, "val7")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zadd("\{pythonsdk_z002\}:004", 800.21, "val8")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zadd("\{pythonsdk_z002\}:004", 900.21, "val9")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zadd("\{pythonsdk_z002\}:004", 1000.21, "val10")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zadd("\{pythonsdk_z002\}:004", 1100.21, "val11")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zremrangebyrank("\{pythonsdk_z002\}:004", 2, 3)

# result = redis_module.redis_clients["pysdk_resource_id_02"].zremrangebyscore("\{pythonsdk_z002\}:004", 1000, 1200)

# result = redis_module.redis_clients["pysdk_resource_id_02"].zrevrange("\{pythonsdk_z002\}:004", 0, -1, "withscores")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zrevrangebyscore("\{pythonsdk_z002\}:004", 1200, 0)

# result = redis_module.redis_clients["pysdk_resource_id_02"].zrevrank("\{pythonsdk_z002\}:004", "val7")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zrevrank("\{pythonsdk_z002\}:004", "val7")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zscore("\{pythonsdk_z002\}:004", "val8")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zunionstore("\{pythonsdk_z002\}:006", 2, "\{pythonsdk_z002\}:001", "\{pythonsdk_z002\}:002")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zrevrange("\{pythonsdk_z002\}:006", 0, -1, "withscores")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zinterstore("\{pythonsdk_z002\}:007", 2, "\{pythonsdk_z002\}:001", "\{pythonsdk_z002\}:002")

# result = redis_module.redis_clients["pysdk_resource_id_02"].zrevrange("\{pythonsdk_z002\}:007", 0, -1, "withscores")
# result = redis_module.redis_clients["49d0eebf34d540a788377135b11bfb7f"].zrevrange("\{pythonsdk_z002\}:007", 0, -1, "withscores")

# result = redis_module.redis_clients["49d0eebf34d540a788377135b11bfb7f"].zadd("\{pythonsdk_z002\}:004", 1100.21, "val11")

# result = redis_module.redis_clients["49d0eebf34d540a788377135b11bfb7f"].zadd("\{pythonsdk_z002\}:004", 1200.21, "val12")

# result = redis_module.redis_clients["49d0eebf34d540a788377135b11bfb7f"].zrangebyscore("\{pythonsdk_z002\}:004", 0, 1200)

# redis_module.redis.set_timeout((10, 20))
# result = redis_module.redis.zrangebyscore("\{pythonsdk_z002\}:004", 0, 1200)

# result = redis_module.redis.mset("\{same_slot\}:commonset002", "val002","\{same_slot\}:commonset003", "val003")

# result = redis_module.redis.mget("\{same_slot\}:commonset002", "\{same_slot\}:commonset002.5","\{same_slot\}:commonset003", "\{same_slot\}:commonset004")

# print("result", result)



import typing as _typing
from .remote import file_api as _api
from .remote.config import get_baas_file_resource_id as _get_baas_file_resource_id

def upload(name:str, file:_typing.Union[bytes, _typing.BinaryIO], timeout:_typing.Union[float, tuple]=10)->_api.File:
  """Upload file.

  :param file: bytes of a file content, or a file-like object, be aware of file size, if limits exceed, exception will be raised.
  :param timeout: (optional) timeout in seconds for the request. if it is a tuple, the first number will be connection timeout, 
    while the second will be read timeout.
  :return: :class:`_api <File>` object
  :rtype: _api.File
  """
  data = None
  if isinstance(file, str):  # 如果是文件名
    with open(file, 'rb') as f:
      data = f.read()
  elif isinstance(file, bytes): # 如果是字节流
    data = file
  elif hasattr(file, 'read'):  # 如果是文件对象
    data = file.read()
  else:
    raise Exception("file type not support")
  resp = _api.presign_upload_file(data, name, _get_baas_file_resource_id(), timeout)
  upload_resp = _api.get_s3_session().put(resp.url, data=data, headers=resp.additional_header, timeout=_api.get_s3_timeout(timeout))
  upload_resp.raise_for_status()
  return resp.file

def download(file_id:str, timeout:_typing.Union[float, tuple]=10):
  """Download file.

  :param file_id: File.id returned by upload function.
  :param timeout: (optional) timeout in seconds for the request. if it is a tuple, the first number will be connection timeout, 
    while the second will be read timeout.
  :return: None
  :rtype: None
  :raises FileNotFound: if provided file id is not found.
  """
  file_desc = _api.get_file_info(file_id, _get_baas_file_resource_id(), timeout)
  if file_desc is None:
    raise FileNotFound
  download_url = file_desc.url
  resp = _api.get_s3_session().get(download_url, timeout=_api.get_s3_timeout(timeout))
  return resp.content

def delete(file_id:str, timeout:_typing.Union[float, tuple]=10):
  """Delete file.

  :param file_id: File.id returned by upload function.
  :param timeout: (optional) timeout in seconds for the request. if it is a tuple, the first number will be connection timeout, 
    while the second will be read timeout.
  :return: None
  :rtype: None
  """
  return _api.delete(file_id, _get_baas_file_resource_id(), timeout)

class FileNotFound(Exception):...


import requests
import json
import os
import a0_baas_sdk.remote.auth as auth
from . import utils
from typing import Union as _Union,Tuple as _Tuple


def _get_redis_basic_url(host: str, resource_id: str):
  return f"{host}/v1/baas/data/redis/cmd/{resource_id}/"


_SYSTEM_ERROR_CODE = "1220002"


def _parse_redis_response(resp:requests.Response):
  res = utils._parse_response(resp)
  try:
    if "JSONStr" not in res:
      raise utils.RemoteError(_SYSTEM_ERROR_CODE, resp.status_code, f"unexpected response, status code: {resp.status_code}, message: {resp.text}", None)
    json_val = json.loads(res["JSONStr"])
    return json_val
  except:
    raise utils.RemoteError(_SYSTEM_ERROR_CODE, resp.status_code, f"unexpected response, status code: {resp.status_code}, message: {resp.text}", None)

def redis_query(host: str, resource_id: str, timeout: _Union[float, _Tuple[float, float]], redis_method: str, args):
  args = [str(arg) for arg in args]
  json_data = json.dumps({"Args": args})
  auth_header = auth.get_auth_header(resource_id)
  headers = {
    "Content-Type": "application/json"
  }
  headers.update(auth_header)
  resp = requests.post(_get_redis_basic_url(host, resource_id)+redis_method, json_data, headers=headers, timeout=_get_timeout(timeout))
  return _parse_redis_response(resp)


def _get_timeout(timeout: _Union[float, _Tuple[float, float]]):
  if isinstance(timeout, tuple):
    return timeout
  else:
    return (5, timeout)




import requests
import time

_SYSTEM_ERROR_CODE = "0200001"

def _parse_response(resp:requests.Response):
  try:
    ret = resp.json()
  except Exception:
    raise RemoteError(_SYSTEM_ERROR_CODE, resp.status_code, f"unexpected response, status code: {resp.status_code}, message: {resp.text}", None)
  if ("ResponseMetadata" not in ret) or (ret["ResponseMetadata"] == None) or ("Error" not in ret["ResponseMetadata"]):
    raise RemoteError(_SYSTEM_ERROR_CODE, resp.status_code, f"unexpected response without meta data field, status code: {resp.status_code}, message: {resp.text}", None)
  if (ret["ResponseMetadata"]["Error"] == None) or (ret["ResponseMetadata"]["Error"]["Code"]==""):
    if "Result" in ret:
      return ret["Result"]
    else:
      raise RemoteError(_SYSTEM_ERROR_CODE, resp.status_code, f"unexpected response without result field, status code: {resp.status_code}, message: {resp.text}", None)
  if ret["ResponseMetadata"]["Error"]["Code"]!="":
    err = ret["ResponseMetadata"]["Error"]
    raise RemoteError(err["Code"], resp.status_code, err["Message"], err["Data"])
  else:
    raise RemoteError(_SYSTEM_ERROR_CODE, resp.status_code, f"unexpected response, status code: {resp.status_code}, message: {resp.text}", None)

class RemoteError(Exception):
  """
  Error From Remote Server
  """
  code = None
  http_status = None
  message = None
  data = None
  def __init__(self, code:str, http_code:str, message:str, data:dict, *args, **kwargs):
    self.code = code
    self.http_code = http_code
    self.message = message
    self.data = data
    super().__init__(*args, **kwargs)
  def __str__(self) -> str:
    return f"remote error, http_status: {self.http_code}, code: {self.code}, message: {self.message}"

def retry(times:int, decay:float):
  def inner(f):
    def wraped(*args, **kwarg):
      for i in range(0, times+1):
        try:
          return f(*args, **kwarg)
        except Exception as e:
          if i == times:
            raise e
          time.sleep(decay)
    return wraped
  return inner  


import os
import hashlib
import base64
from typing import Tuple, Union
import requests
import a0_baas_sdk.remote.auth as auth
from . import utils
from .config import get_baas_server_host, get_baas_file_resource_id
from urllib3.util import Retry
from requests import Session
from requests.adapters import HTTPAdapter


class File(object):
  id: str
  name: str
  size: int
  checksum_md5: str
  url: str
  def __init__(self, id, name, size, url) -> None:
    self.id=id
    self.name=name
    self.size=size
    self.url=url

class PresignResult(object):
  url:str
  file:File
  additional_header:dict
  def __init__(self, url, file, additional_header) -> None:
    self.url=url
    self.file=file
    self.additional_header=additional_header

def _get_upload_presign_url(resource_id:str):
  return f"{get_baas_server_host()}/v1/baas/data/file/api/resource/{resource_id}/put/presign"

def _get_delete_url(resource_id:str,file_id:str):
  return f"{get_baas_server_host()}/v1/baas/data/file/api/resource/{resource_id}/file/{file_id}"

def _get_get_url(resource_id:str,file_id:str):
  return f"{get_baas_server_host()}/v1/baas/data/file/api/resource/{resource_id}/file/{file_id}"

def presign_upload_file(data:bytes, name:str,resource_id:str, timeout:Union[float, Tuple[float]])->PresignResult:
  checksum = hashlib.md5(data).digest()
  checksum_base64 = base64.b64encode(checksum).decode('utf-8')
  url = _get_upload_presign_url(resource_id)
  resp = _get_baas_session().get(url, data={
    "Name": name,
    "Size": len(data),
    "CheckSumMD5":checksum_base64
  },headers=auth.get_auth_header(resource_id), timeout=_get_timeout(timeout))
  presign_resp = utils._parse_response(resp)
  file_desc = presign_resp["File"]
  file = File(file_desc["ID"], file_desc["Name"], file_desc["Size"], file_desc["URL"])
  return PresignResult(presign_resp["URL"], file, presign_resp["AdditionalHeader"])

def delete(file_id:str,resource_id:str, timeout:Union[float, Tuple[float, float]]=10)->None:
  """
  删除文件
  """
  url = _get_delete_url(resource_id,file_id)
  resp = _get_baas_session().delete(url, headers=auth.get_auth_header(resource_id), timeout=_get_timeout(timeout))
  return utils._parse_response(resp)

def get_file_info(file_id:str,resource_id:str, timeout:Union[float, Tuple[float]])->File:
  """
  通过 file_id 下载文件
  """
  url = _get_get_url(resource_id, file_id)
  resp = _get_baas_session().get(url, headers=auth.get_auth_header(resource_id), timeout=_get_timeout(timeout))
  file_resp = utils._parse_response(resp)
  if file_resp is None:
    return None
  return File(file_resp["ID"], file_resp["Name"], file_resp["Size"], file_resp["URL"])

def get_s3_session()->Session:
  s = Session()
  return s
  # retries = Retry(
  #     total=3,
  #     backoff_factor=0.1,
  #     status_forcelist=[502, 503, 504],
  #     allowed_methods={'POST'},
  # )
  # s.mount('http://', HTTPAdapter(max_retries=retries))
  # s.mount('https://', HTTPAdapter(max_retries=retries))

def _get_baas_session()->Session:
  s = Session()
  return s

def _get_timeout(timeout):
  if isinstance(timeout, tuple):
    return timeout
  else:
    return (0.1, timeout)

def get_s3_timeout(timeout):
  if isinstance(timeout, tuple):
    return timeout
  else:
    return (0.1, timeout)
  


from typing import Union as _Union,List as _List,Tuple as _Tuple
# import sys
# sys.path.append(r"/cloudide/workspace/baas-sdk-python/src/")
from .remote.redis_api import redis_query as _redis_query
from .remote.config import get_baas_redis_resource_arr as _get_baas_redis_resource_arr,get_baas_server_host as _get_baas_server_host, get_baas_redis_server_host as _get_baas_redis_server_host


class _RedisClass:
  def __init__(self, host: str, resource_id: str):
    self.host = host
    self.resource_id = resource_id
    self.timeout = 10
  def set_timeout(self, timeout: _Union[float, _Tuple[float, float]]):
    self.timeout = timeout
  def delete(self, key: str, *other_keys: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "del", [key, *other_keys])
  def exists(self, key: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "exists", [key])
  def expire(self, key: str, seconds: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "expire", [key, seconds])
  def expireat(self, key: str, timestamp: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "expireat", [key, timestamp])
  def persist(self, key: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "persist", [key])
  def pexpire(self, key: str, milliseconds: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "pexpire", [key, milliseconds])
  def pexpireat(self, key: str, timestamp: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "pexpireat", [key, timestamp])
  def pexpireat(self, key: str, timestamp: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "pexpireat", [key, timestamp])
  def pttl(self, key: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "pttl", [key])
  def ttl(self, key: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "ttl", [key])
  def type(self, key: str) -> str:
    return _redis_query(self.host, self.resource_id, self.timeout, "type", [key])
  def append(self, key: str, value: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "append", [key, value])
  def bitcount(self, key: str, start: int, end: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "bitcount", [key, start, end])
  def decr(self, key: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "decr", [key])
  def decrby(self, key: str, num: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "decrby", [key, num])
  def get(self, key: str) -> _Union[str, None]:
    return _redis_query(self.host, self.resource_id, self.timeout, "get", [key])
  def getbit(self, key: str, offset: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "getbit", [key, offset])
  def getrange(self, key: str, start: int, end: int) -> str:
    return _redis_query(self.host, self.resource_id, self.timeout, "getrange", [key, start, end])
  def getset(self, key: str, new_value: str) -> str:
    return _redis_query(self.host, self.resource_id, self.timeout, "getset", [key, new_value])
  def incr(self, key: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "incr", [key])
  def incrby(self, key: str, num: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "incrby", [key, num])
  def incrbyfloat(self, key: str, num: float) -> float:
    return _redis_query(self.host, self.resource_id, self.timeout, "incrbyfloat", [key, num])
  def mget(self, key: str, *other_keys: str) -> _List[_Union[str, None]]:
    return _redis_query(self.host, self.resource_id, self.timeout, "mget", [key, *other_keys])
  def psetex(self, key: str, milliseconds: int, value: str) -> str:
    return _redis_query(self.host, self.resource_id, self.timeout, "psetex", [key, milliseconds, value])
  def set(self, key: str, value: str, set_type_or_expire_mode: _Union[str, None]=None, expire: _Union[int, None]=None, set_type: _Union[str, None] = None) -> _Union[str, None]:
    """
    usage:
    set(key: str, value: str)
    set(key: str, value: str, set_type: 'nx' | 'xx')
    set(key: str, value: str, expire_mode: 'ex' | 'px', expire: int)
    set(key: str, value: str, expire_mode: 'ex' | 'px', expire: int, set_type: 'nx' | 'xx')
    """
    input = tuple(filter(lambda x: x is not None, [key, value, set_type_or_expire_mode, expire, set_type]))
    return _redis_query(self.host, self.resource_id, self.timeout, "set", input)
  def mset(self, key: str, value: str, *key_or_value: str) -> str:
    return _redis_query(self.host, self.resource_id, self.timeout, "mset", [key, value, *key_or_value])
  def setbit(self, key: str, offset: int, value: int) -> str:
    """
    usage:
    setbit(key: str, offset: int, value: 0 | 1)
    """
    return _redis_query(self.host, self.resource_id, self.timeout, "setbit", [key, offset, value])
  def setex(self, key: str, seconds: int, value: str) -> str:
    return _redis_query(self.host, self.resource_id, self.timeout, "setex", [key, seconds, value])
  def setnx(self, key: str, value: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "setnx", [key, value])
  def setrange(self, key: str, offset: int, value: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "setrange", [key, offset, value])
  def strlen(self, key: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "strlen", [key])
  def hdel(self, key: str, value: str, *other_values: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "hdel", [key, value, *other_values])
  def hexists(self, key: str, field: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "hexists", [key, field])
  def hget(self, key: str, field: str) -> _Union[str, None]:
    return _redis_query(self.host, self.resource_id, self.timeout, "hget", [key, field])
  def hgetall(self, key: str) -> dict[str, str]:
    return _redis_query(self.host, self.resource_id, self.timeout, "hgetall", [key])
  def hincrby(self, key: str, field: str, increment: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "hincrby", [key, field, increment])
  def hincrbyfloat(self, key: str, field: str, increment: float) -> float:
    return _redis_query(self.host, self.resource_id, self.timeout, "hincrbyfloat", [key, field, increment])
  def hkeys(self, key: str) -> _List[str]:
    return _redis_query(self.host, self.resource_id, self.timeout, "hkeys", [key])
  def hlen(self, key: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "hlen", [key])
  def hmget(self, key: str, field: str, *other_field: str) -> _List[_Union[str, None]]:
    return _redis_query(self.host, self.resource_id, self.timeout, "hmget", [key, field, *other_field])
  def hmset(self, key: str, field:str, value:str, *field_or_value: str) -> str:
    """
    hmset(key: str, field1: str, value1: str, field2: str, value2: str, ...)
    """
    return _redis_query(self.host, self.resource_id, self.timeout, "hmset", [key, field, value, *field_or_value])
  def hset(self, key: str, field: str, value: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "hset", [key, field, value])
  def hsetnx(self, key: str, field: str, value: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "hsetnx", [key, field, value])
  def hvals(self, key: str) -> _List[str]:
    return _redis_query(self.host, self.resource_id, self.timeout, "hvals", [key])
  def lindex(self, key: str, index: int) -> _Union[str, None]:
    return _redis_query(self.host, self.resource_id, self.timeout, "lindex", [key, index])
  def llen(self, key: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "llen", [key])
  def lpop(self, key: str) -> _Union[str, None]:
    return _redis_query(self.host, self.resource_id, self.timeout, "lpop", [key])
  def lpush(self, key: str, field: str, *other_fields: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "lpush", [key, field, *other_fields])
  def lpushx(self, key: str, field: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "lpushx", [key, field])
  def lrange(self, key: str, start: int, end: int) -> _List[str]:
    return _redis_query(self.host, self.resource_id, self.timeout, "lrange", [key, start, end])
  def lset(self, key: str, index: int, value: str) -> str:
    return _redis_query(self.host, self.resource_id, self.timeout, "lset", [key, index, value])
  def ltrim(self, key: str, start: int, end: str) -> str:
    return _redis_query(self.host, self.resource_id, self.timeout, "ltrim", [key, start, end])
  def rpop(self, key: str) -> _Union[str, None]:
    return _redis_query(self.host, self.resource_id, self.timeout, "rpop", [key])
  def rpoplpush(self, source: str, destination: str) -> _Union[str, None]:
    return _redis_query(self.host, self.resource_id, self.timeout, "rpoplpush", [source, destination])
  def rpush(self, key: str, field: str, *other_fields: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "rpush", [key, field, *other_fields])
  def rpushx(self, key: str, field: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "rpushx", [key, field])
  def sadd(self, key: str, field: str, *other_fields: str) -> int:
    """
    if want key be in same slot
    sadd("\{pythonsdk_set002\}:somekey", "one","two","zero","two")
    """
    return _redis_query(self.host, self.resource_id, self.timeout, "sadd", [key, field, *other_fields])
  def sdiff(self, key: str, *other_keys: str) -> _List[str]:
    return _redis_query(self.host, self.resource_id, self.timeout, "sdiff", [key, *other_keys])
  def sdiffstore(self, destination: str, key: str, *other_keys: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "sdiffstore", [destination, key, *other_keys])
  def sinter(self, key: str, *other_keys: str) -> _List[str]:
    return _redis_query(self.host, self.resource_id, self.timeout, "sinter", [key, *other_keys])
  def sinterstore(self, destination: str, key: str, *other_keys: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "sinterstore", [destination, key, *other_keys])
  def sismember(self, key: str, member: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "sismember", [key, member])
  def smembers(self, key: str) -> _List[str]:
    return _redis_query(self.host, self.resource_id, self.timeout, "smembers", [key])
  def smove(self, source: str, destination: str, member: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "smove", [source, destination, member])
  def spop(self, key: str, count: _Union[int, None] = None) -> _List[str]:
    input = tuple(filter(lambda x: x is not None, [key, count]))
    return _redis_query(self.host, self.resource_id, self.timeout, "spop", input)
  def srandmember(self, key: str, count: _Union[int, None] = None) -> _List[str]:
    input = tuple(filter(lambda x: x is not None, [key, count]))
    return _redis_query(self.host, self.resource_id, self.timeout, "srandmember", input)
  def srem(self, key: str, field: str, *other_fields: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "srem", [key, field, *other_fields])
  def sunion(self, key: str, *other_keys: str) -> _List[str]:
    return _redis_query(self.host, self.resource_id, self.timeout, "sunion", [key, *other_keys])
  def sunionstore(self, destination: str, key: str, *other_keys: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "sunionstore", [destination, key, *other_keys])
  def zadd(self, key: str, score: float, member: str,*other_score_or_member: _Union[str, int]) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "zadd", [key, score, member, *other_score_or_member])
  def zcard(self, key: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "zcard", [key])
  def zcount(self, key: str, min: _Union[str, int], max: _Union[str, int]) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "zcount", [key, min, max])
  def zincrby(self, key: str, increment: float, member: str) -> float:
    return _redis_query(self.host, self.resource_id, self.timeout, "zincrby", [key, increment, member])
  def zrange(self, key: str, start: int, end: int, withscores: _Union[str, None] = None) -> _List[str]:
    """
    zrange(key: str, start: int, end: int, withscores: 'withscores')
    """
    input = tuple(filter(lambda x: x is not None, [key, start, end, withscores]))
    return _redis_query(self.host, self.resource_id, self.timeout, "zrange", input)
  def zrangebyscore(self, key: str, min: _Union[str, int], max: _Union[str, int]) -> _List[str]:
    return _redis_query(self.host, self.resource_id, self.timeout, "zrangebyscore", [key, min, max])
  def zrank(self, key: str, member: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "zrank", [key, member])
  def zrem(self, key: str, member: str, *other_members: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "zrem", [key, member, *other_members])
  def zremrangebyrank(self, key: str, start: int, end: int) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "zremrangebyrank", [key, start, end])
  def zremrangebyscore(self, key: str, min: _Union[str, int], max: _Union[str, int]) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "zremrangebyscore", [key, min, max])
  def zrevrange(self, key: str, start: int, end: int, withscores: _Union[str, None] = None) -> _List[str]:
    """
    zrevrange(key: str, start: int, end: int, withscores: 'withscores')
    """
    input = tuple(filter(lambda x: x is not None, [key, start, end, withscores]))
    return _redis_query(self.host, self.resource_id, self.timeout, "zrevrange", input)
  def zrevrangebyscore(self, key: str, max: _Union[str, int], min: _Union[str, int]) -> _List[str]:
    return _redis_query(self.host, self.resource_id, self.timeout, "zrevrangebyscore", [key, max, min])
  def zrevrank(self, key: str, member: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "zrevrank", [key, member])
  def zscore(self, key: str, member: str) -> float:
    return _redis_query(self.host, self.resource_id, self.timeout, "zscore", [key, member])
  def zinterstore(self, destination: str, key_num: int, key: str, *other_keys: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "zinterstore", [destination, key_num, key, *other_keys])
  def zunionstore(self, destination: str, key_num: int, key: str, *other_keys: str) -> int:
    return _redis_query(self.host, self.resource_id, self.timeout, "zunionstore", [destination, key_num, key, *other_keys])

redis: _RedisClass
redis_clients: dict[str, _RedisClass] = {}

_resource_arr = _get_baas_redis_resource_arr()
_baas_host = _get_baas_server_host()
_inner_redis_host = _get_baas_redis_server_host()

_redis_host = ""
if _inner_redis_host!="":
  _redis_host = _inner_redis_host
else:
  _redis_host = _baas_host


for _resource in _resource_arr:
  redis_clients[_resource.id] = _RedisClass(_redis_host, _resource.id)

if len(_resource_arr) > 0:
  redis = redis_clients[_resource_arr[0].id]
